// -*- Mode: C++; -*-
//                            Package   : omniEvents
// eventc.cc                  Created   : 1/4/98
//                            Author    : Paul Nader (pwn)
//
//    Copyright (C) 1998 Paul Nader.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Description:
//    Client to the event channel factory. Requests creation of an event
//    channel and registers it with the Naming service.
//	

/*
  $Log: eventc.cc,v $
  Revision 1.3  2003/11/03 22:21:21  alextingle
  Removed all platform specific switches. Now uses autoconf, config.h.
  Removed stub header in order to allow makefile dependency checking to work
  correctly.

  Revision 1.1.1.1.2.1  2002/09/28 22:20:51  shamus13
  Added ifdefs to enable omniEvents to compile
  with both omniORB3 and omniORB4. If __OMNIORB4__
  is defined during compilation, omniORB4 headers
  and command line option syntax is used, otherwise
  fall back to omniORB3 style.

  Revision 1.1.1.1  2002/09/25 19:00:25  shamus13
  Import of OmniEvents source tree from release 2.1.1

  Revision 1.6  2000/09/05 01:05:38  naderp
  Added MaxQueueLength QOS.

  Revision 1.5  2000/08/30 04:39:20  naderp
  Port to omniORB 3.0.1.

  Revision 1.4  2000/03/16 05:34:30  naderp
  Added stdlib.h for solaris getopt()

  Revision 1.3  2000/03/16 02:44:13  naderp
  Added iostream and signal headers.

  Revision 1.2  2000/03/06 13:23:50  naderp
  Using util getRootNamingContext function.
  Using stub headers.

  Revision 1.1  1999/11/01 20:37:42  naderp
  Updated usage statement.

Revision 1.0  99/11/01  17:05:13  17:05:13  naderp (Paul Nader)
omniEvents 2.0.
Added -m switch to support MaxEventsPerConsumer criteria.

Revision 0.6  99/08/27  11:48:22  11:48:22  naderp (Paul Nader)
Partitioned EventChannelFactory_i from CosEvent_i.

Revision 0.5  99/05/10  11:27:50  11:27:50  naderp (Paul Nader)
Initialised rootContext.

Revision 0.4  99/04/23  16:02:22  16:02:22  naderp (Paul Nader)
gcc port.

Revision 0.3  99/04/23  09:32:58  09:32:58  naderp (Paul Nader)
Windows Port.

Revision 0.2  99/04/21  18:06:23  18:06:23  naderp (Paul Nader)
*** empty log message ***

Revision 0.1.1.1  98/11/27  17:01:51  17:01:51  naderp (Paul Nader)
Enclosed supports call in try block to avoid core dump.
Added information messages for exceptions.

Revision 0.1  98/11/25  14:07:22  14:07:22  naderp (Paul Nader)
Initial Revision

*/

//
//

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_GETOPT
#  include <unistd.h>
extern char* optarg;
extern int optind;
#else
#  include "getopt.h"
#endif

#ifdef HAVE_IOSTREAM
#  include <iostream>
#else
#  include <iostream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif

#ifdef HAVE_SIGNAL_H
#  include <signal.h>
#endif

#include "naming.h"

#include "CosEventChannelAdmin.hh"
#include "EventChannelAdmin.hh"

static void usage();

int
main(int argc, char **argv)
{
  int result =1;

  //
  // Start orb and boa.
#if defined(HAVE_OMNIORB4)
  CORBA::ORB_ptr orb = CORBA::ORB_init(argc,argv,"omniORB4");
  CORBA::BOA_ptr boa = orb->BOA_init(argc,argv,"omniORB4_BOA");
#else
  CORBA::ORB_ptr orb = CORBA::ORB_init(argc,argv,"omniORB3");
  CORBA::BOA_ptr boa = orb->BOA_init(argc,argv,"omniORB3_BOA");
#endif

  // Process Options
  int c;
  const char*   channelName          ="EventChannel";
  const char*   channelKind          ="EventChannel";
  const char*   factoryName          ="EventChannelFactory";
  const char*   factoryKind          ="EventChannelFactory";
  CORBA::ULong  pullRetryPeriod      =1;
  CORBA::ULong  maxQueueLength       =0;
  CORBA::ULong  maxEventsPerConsumer =0;

  while ((c = getopt(argc,argv,"n:k:N:K:r:m:q:h")) != EOF)
  {
     switch (c)
     {
        case 'n': channelName = optarg;
                  break;

        case 'k': channelKind = optarg;
                  break;

        case 'N': factoryName = optarg;
                  break;

        case 'K': factoryKind = optarg;
                  break;

        case 'r': pullRetryPeriod = atol(optarg);
                  break;

        case 'm': maxEventsPerConsumer = atol(optarg);
                  break;

        case 'q': maxQueueLength = atol(optarg);
                  break;

        case 'h':
        default : usage();
                  exit(-1);
                  break;
     }
  }

  //
  // Use one big try...catch block.
  // 'action' variable keeps track of what we're doing.
  const char* action ="start";
  try
  {

    //
    // Get Name Service root context.
    action="get NameService initial reference";
    CosNaming::NamingContext_ptr rootContext;
    rootContext = getRootNamingContext(orb);

    //
    // Obtain reference to the Event Channel Factory implementation.
    action="find Event Channel Factory in naming service";
    CosNaming::Name name;
    name.length (1);
    name[0].id   = CORBA::string_dup(factoryName);
    name[0].kind = CORBA::string_dup(factoryKind);

    CORBA::Object_var obj = rootContext->resolve(name);
    EventChannelAdmin::EventChannelFactory_var factory =
      EventChannelAdmin::EventChannelFactory::_narrow(obj);
    if(CORBA::is_nil(factory))
    {
       cerr << "Failed to narrow Event Channel Factory reference." << endl;
       exit(1);
    }

    // Check that the factory is of the right type
    action="check factory supports EventChannel object interface";
    CosLifeCycle::Key key;
    key.length (1);
    key[0].id = CORBA::string_dup("EventChannel");
    key[0].kind = CORBA::string_dup("object interface");

    if(!factory->supports(key))
    {
      cerr << "Factory does not support Event Channel Interface ! [\""
           << factoryName << "\", \"" << factoryKind << "\"]"
           << endl;
      exit (1);
    }

    //
    // Create Event Channel Object.
    action="create EventChannel object";

    // Construct criteria
    CosLifeCycle::Criteria criteria;
    criteria.length(3);
    criteria[0].name = CORBA::string_dup ("PullRetryPeriod");
    criteria[0].value <<= (CORBA::ULong) pullRetryPeriod;
    criteria[1].name = CORBA::string_dup ("MaxEventsPerConsumer");
    criteria[1].value <<= (CORBA::ULong) maxEventsPerConsumer;
    criteria[2].name = CORBA::string_dup ("MaxQueueLength");
    criteria[2].value <<= (CORBA::ULong) maxQueueLength;

    CORBA::Object_var channelObj =factory->create_object(key, criteria);
    if (CORBA::is_nil(channelObj))
    {
       cerr << "Channel Factory returned nil reference! [\""
            << channelName << "\", \"" << channelKind << "\"]" << endl;
       exit(1);
    }

    // Narrow object returned to an Event Channel
    CosEventChannelAdmin::EventChannel_var channel =
      CosEventChannelAdmin::EventChannel::_narrow(channelObj);
    if (CORBA::is_nil(channel))
    {
       cerr << "Failed to narrow Event Channel ! [\""
            << channelName << "\", \"" << channelKind << "\"]"
            << endl;
       exit(1);
    }

    //
    // Register event channel with naming service
    name.length(1);
    name[0].id   = CORBA::string_dup (channelName);
    name[0].kind = CORBA::string_dup (channelKind);

    try{
      action="register (bind) EventChannel with the naming service";
      rootContext->bind(name,channel.in());
    }
    catch(CosNaming::NamingContext::AlreadyBound& ex) {
      action="register (rebind) EventChannel with the naming service";
      rootContext->rebind(name,channel.in());
    }
    
    //
    // If we get here, then everything has worked OK.
    result=0;

  }
  catch (CosLifeCycle::NoFactory& ex) /* create_object() */ {
     cerr<<"Failed to create Event Channel: NoFactory"
       " (interface not supported) "<<endl;
  }
  catch (CosLifeCycle::CannotMeetCriteria& ex) /* create_object() */ {
     cerr<<"Failed to create Event Channel: CannotMeetCriteria "<<endl;
  }
  catch (CosLifeCycle::InvalidCriteria& ex) /* create_object() */ {
     cerr<<"Failed to create Event Channel: InvalidCriteria "<<endl;
  }
  catch (CORBA::COMM_FAILURE& ex) {
     cerr<<"System exception, unable to "<<action<<": COMM_FAILURE"<<endl;
  }
  catch (CORBA::SystemException& ex) {
     cerr<<"System exception, unable to "<<action
#ifdef HAVE_OMNIORB4
         <<": "<<ex._name()<<" ("<<ex.NP_minorString()<<")"
#endif
         << endl;
  }
  catch (CORBA::Exception& ex) {
     cerr<<"CORBA exception, unable to "<<action
#ifdef HAVE_OMNIORB4
         <<": "<<ex._name()
#endif
         << endl;
  }
  catch (omniORB::fatalException& ex) {
     cerr<<"Fatal Exception, unable to "<<action<<endl;
  }

  return result;
}

static void
usage()
{
   cerr << "\nusage: eventc [-n name] [-k kind] [-N name] [-K kind] [-r sec] [-m events] [-q events] [-h]\n" << endl;
   cerr << "         -n name specify channel name [\"EventChannel\"]" << endl;
   cerr << "         -k kind specify channel kind [\"EventChannel\"]" << endl;
   cerr << "         -N name specify factory name [\"EventChannelFactory\"]" << endl;
   cerr << "         -K kind specify factory kind [\"EventChannelFactory\"]" << endl;
   cerr << "         -r Pull retry Period (in seconds) [1 sec]" << endl;
   cerr << "         -m Maximum Number of Events queued per Consumer [0 = unlimited]" << endl;
   cerr << "         -q Maximum Number of Events queued by the channel [0 = unlimited]" << endl;
   cerr << "         -h     display usage" << endl;
   cerr << endl;
}

