// -*- Mode: C++; -*-
//                            Package   : omniEvents
//   pullsupp.cc              Created   : 1/4/98
//                            Author    : Paul Nader (pwn)
//
//    Copyright (C) 1998 Paul Nader.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Description:
//    Pull Model supplier implementation.
//	

/*
  $Log: pullsupp.cc,v $
  Revision 1.4  2003/11/17 08:47:09  alextingle
  Corrected typo.

  Revision 1.2  2003/11/16 23:24:22  alex
  Corrected typo.

  Revision 1.1.1.1  2003/11/11 22:28:56  alex
  Import of testing 2.3.0

  Revision 1.3  2003/11/03 22:20:26  alextingle
  Removed all platform specific switches. Now uses autoconf, config.h.
  Removed stub header in order to allow makefile dependency checking to work
  correctly.

  Revision 1.1.1.1.2.1  2002/09/28 22:20:51  shamus13
  Added ifdefs to enable omniEvents to compile
  with both omniORB3 and omniORB4. If __OMNIORB4__
  is defined during compilation, omniORB4 headers
  and command line option syntax is used, otherwise
  fall back to omniORB3 style.

  Revision 1.1.1.1  2002/09/25 19:00:26  shamus13
  Import of OmniEvents source tree from release 2.1.1

  Revision 0.11  2000/08/30 04:39:48  naderp
  Port to omniORB 3.0.1.

  Revision 0.10  2000/03/16 05:37:27  naderp
  Added stdlib.h for getopt.

  Revision 0.9  2000/03/06 13:26:32  naderp
  Using util getRootNamingContext function.
  Using stub headers.
  Fixed error messages.

  Revision 0.8  2000/03/02 03:16:26  naderp
  Added retry resiliency for handling COMM_FAUILURE exceptions.
  Replaced condition variable by counting semaphore.

  Revision 0.7  1999/11/02 13:39:13  naderp
  Added <signal.h>

  Revision 0.6  1999/11/02 07:57:22  naderp
  Updated usage.

Revision 0.5  99/11/01  16:10:12  16:10:12  naderp (Paul Nader)
omniEvents 2.0 Release.
Ignoring SIGPIPE for UNIX platforms.

Revision 0.4  99/04/23  16:05:44  16:05:44  naderp (Paul Nader)
gcc port.

Revision 0.3  99/04/23  09:34:01  09:34:01  naderp (Paul Nader)
Windows Port.

Revision 0.2  99/04/21  18:06:25  18:06:25  naderp (Paul Nader)
*** empty log message ***

Revision 0.1.1.1  98/11/27  16:59:33  16:59:33  naderp (Paul Nader)
Added -s option to sleep after disconnecting.

Revision 0.1  98/11/25  14:08:07  14:08:07  naderp (Paul Nader)
Initial Revision

*/

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_GETOPT
#  include <unistd.h>
extern char* optarg;
extern int optind;
#else
#  include "getopt.h"
#endif

#ifdef HAVE_IOSTREAM
#  include <iostream>
#else
#  include <iostream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif

#ifdef HAVE_SIGNAL_H
#  include <signal.h>
#endif

#include "naming.h"

#include "CosEventComm.hh"
#include "CosEventChannelAdmin.hh"

static omni_semaphore connect_cond(0);
static void usage();

class Supplier_i : virtual public CosEventComm::_sk_PullSupplier {
public:
  Supplier_i (long disconnect = 0) : i(0), _disconnect(disconnect), l(0) {};
  CORBA::Any *pull();
  CORBA::Any *try_pull(CORBA::Boolean &has_event);
  void disconnect_pull_supplier ();

private:
  long i;
  long _disconnect;
  CORBA::ULong l;
};

void
Supplier_i::disconnect_pull_supplier () {
  cout << "Pull Supplier: disconnected by channel." << endl;
}

CORBA::Any *
Supplier_i::pull() {
   cout << "Pull Supplier: pull() called. Data : ";
   CORBA::Any *any = new CORBA::Any();
   *any <<= l++;
   cout << l-1 << endl;

   // Exersize Disconnect
   if ((_disconnect > 0) && (i == _disconnect)) {
      i = 0;
      // Signal main thread to disconnect and re-connect.
      connect_cond.post();
   }
   i++;
   return (any);
}

CORBA::Any *
Supplier_i::try_pull(CORBA::Boolean &has_event)
{
   cout << "Pull Supplier: pull() called. Data : ";
   CORBA::Any *any = new CORBA::Any();
   *any <<= l++;
   cout << l-1 << endl;
   has_event = 1;

   // Exersize Disconnect
   if ((_disconnect > 0) && (i == _disconnect)) {
      i = 0;
      // Signal main thread to disconnect and re-connect.
      connect_cond.post();
   }
   i++;
   return (any);
}


int
main (int argc, char** argv)
{
#if defined(HAVE_OMNIORB4)
  CORBA::ORB_ptr orb = CORBA::ORB_init(argc,argv,"omniORB4");
  CORBA::BOA_ptr boa = orb->BOA_init(argc,argv,"omniORB4_BOA");
#else
  CORBA::ORB_var orb = CORBA::ORB_init (argc, argv, "omniORB3");
  CORBA::BOA_var boa = orb->BOA_init (argc, argv, "omniORB3_BOA");
#endif

  // Process Options
  int c;
  int discnum = 0;
  int sleepInterval = 0;
  char *channelName = (char *) "EventChannel";
  char *channelKind = (char *) "EventChannel";
  while ((c = getopt(argc,argv,"d:s:n:k:h")) != EOF)
  {
     switch (c)
     {
        case 'd': discnum = atoi(optarg);
                  break;

        case 's': sleepInterval = atoi(optarg);
                  break;

        case 'n': channelName = optarg;
                  break;

        case 'k': channelKind = optarg;
                  break;

        case 'h':
        default : usage();
                  exit(-1);
                  break;
     }
  }

#ifdef HAVE_SIGNAL_H
  // Ignore broken pipes
  signal(SIGPIPE, SIG_IGN);
#endif

  Supplier_i* supplier = new Supplier_i (discnum);
  supplier->_obj_is_ready(boa);

  // Let  the BOA know we are ready without blocking.
  boa->impl_is_ready(0,1);

  //
  // Obtain and narrow reference to the Name Service.
  CosNaming::NamingContext_var rootContext;
  rootContext = getRootNamingContext(orb);

  // Obtain Event Channel Object.
  CosNaming::Name name;
  name.length (1);
  name[0].id = CORBA::string_dup (channelName);
  name[0].kind = CORBA::string_dup (channelKind);

  cout << "Looking for EventChannel" << endl;
  CosEventChannelAdmin::EventChannel_var event_channel;
  try {
    CORBA::Object_var obj = rootContext->resolve (name);
    event_channel = CosEventChannelAdmin::EventChannel::_narrow (obj);
    if (CORBA::is_nil (event_channel))
    {
       cerr << "Failed to narrow Event Channel !" << endl;
       exit(1);
    }
  }
  catch (CORBA::ORB::InvalidName& ex) {
     cerr << "Service required is invalid [does not exist]. !" << endl;
     exit(1);
  }
  catch (CORBA::COMM_FAILURE& ex) {
     cerr << "Caught COMM_FAILURE exception."
          << "Unable to contact the naming service !"
          << endl;
     exit(1);
  }
  catch (omniORB::fatalException& ex) {
     cerr << "Caught Fatal Exception !" << endl;
     throw;
  }
  catch (...) {
     cerr << "Cannot find event channel ! [\""
          << channelName << "\", \"" << channelKind << "\"]"
          << endl;
    exit(1);
  }

  //
  // Get Supplier Admin interface - retrying on Comms Failure.
  CosEventChannelAdmin::SupplierAdmin_var supplier_admin;
  while (1)
  {
     try {
        supplier_admin = event_channel->for_suppliers ();
        if (CORBA::is_nil(supplier_admin))
        {
           cerr << "Event Channel returned nil Supplier Admin. !"
                << endl;
           exit(1);
        }
        break;
     }
     catch (CORBA::COMM_FAILURE& ex) {
        cerr << "Caught COMM_FAILURE exception "
             << "obtaining Supplier Admin !. Retrying... "
             << endl;
        continue;
     }
     catch (...) {
        cerr << "Unexpected System Exception. "
	     << "Failed to obtain Supplier Admin !"
             << endl;
        exit(1);
     }
  }
  cerr << "Obtained SupplierAdmin." << endl;

  while (1)
  {
     //
     // Get proxy consumer - retrying on Comms Failure.
     CosEventChannelAdmin::ProxyPullConsumer_var proxy_consumer;
     while (1)
     {
        try {
           proxy_consumer = supplier_admin->obtain_pull_consumer ();
           if (CORBA::is_nil(proxy_consumer))
           {
              cerr << "Supplier Admin returned nil proxy_consumer !"
                   << endl;
              exit(1);
           }
	   break;
        }
        catch (CORBA::COMM_FAILURE& ex) {
           cerr << "Caught COMM_FAILURE exception "
                << "obtaining Pull Consumer !. Retrying... "
                << endl;
           continue;
        }
        catch (...) {
           cerr << "Unexpected System Exception. "
	        << "Failed to obtain Proxy Consumer !" 
		<< endl;
           exit(1);
        }
     }
     cerr << "Obtained ProxyPullConsumer." << endl;
   
     // Connect Pull Supplier - retrying on Comms Failure.
     CosEventComm::PullSupplier::_duplicate(supplier->_this());
     while (1)
     {
        try {
           proxy_consumer->connect_pull_supplier(supplier->_this());
           break;
        }
        catch (CORBA::BAD_PARAM& ex) {
           cerr << "Caught BAD_PARAM Exception connecting Pull Supplier !"
                << endl;
           exit (1);
        }
        catch (CosEventChannelAdmin::AlreadyConnected& ex) {
           cerr << "Pull Supplier already connected !"
                << endl;
           break;
        }
        catch (CORBA::COMM_FAILURE& ex) {
           cerr << "Caught COMM_FAILURE Exception "
                << "connecting Pull Supplier !. Retrying..."
                << endl;
           continue;
        }
        catch (...) {
           cerr << "Unexpected System Exception. "
                << "Failed to connect Pull Supplier !"
                << endl;
           exit (1);
        }
     }
     cerr << "Connected Pull Supplier." << endl;

     // Wait for indication to disconnect before re-connecting.
     connect_cond.wait();

     // Disconnect - retrying on Comms Failure.
     while (1)
     {
        try {
           proxy_consumer->disconnect_pull_consumer();
           break;
        }
        catch (CORBA::COMM_FAILURE& ex) {
           cerr << "Caught COMM_FAILURE Exception "
                << "disconnecting Pull Supplier !. Retrying... "
                << endl;
           continue;
        }
        catch (...) {
           cerr << "Unexpected System Exception. "
                << "Failed to disconnect Pull Suplier!"
                << endl;
           exit (1);
        }
     }
     cout << "Disconnected Pull Supplier." << endl;

     // Yawn.
     cerr << "Sleeping " << sleepInterval << " Seconds." << endl;
     omni_thread::sleep(sleepInterval);
  }

  // Not Reached
  return (0);
}

static void
usage()
{
   cerr << "\nusage: pullsupp [-d n [-s n]] [-n name] [-k kind] [-h]\n" << endl;
   cerr << "         -d n   disconnect after n pulls" << endl;
   cerr << "         -s n   sleep n Seconds after disconnecting" << endl;
   cerr << "         -n name specify channel name [\"EventChannel\"]" << endl;
   cerr << "         -k kind specify channel kind [\"EventChannel\"]" << endl;
   cerr << "         -h     display usage" << endl;
   cerr << endl;
}
