#                            Package   : omniEvents
# meta.mk                    Created   : 2003/11/01
#                            Author    : Alex Tingle
#
#    Copyright (C) 2003 Alex Tingle.
#
#    This file is part of the omniEvents application.
#
#    omniEvents is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    omniEvents is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Description:
#
#  A rag-bag of meta-actions.
#
#  o  `make -f meta.mk deps' - Uses g++ -MM to update all deps.mk files.
#
#  o  `make -f meta.mk release' - Builds a 'release' tarball that contains all
#          non-generated files (and */deps.mk - for convenience).
#
#  o  `make -f meta.mk dist' - Builds a 'dist' tarball that contains all
#          files needed to configure & build, but none of the autoconf M4 files,
#          nor this file.
#

include config.mk

DIRNAME = $(shell basename `pwd`)

SUBDIRS = idl src examples
DEP_FILES = $(patsubst %,%/deps.mk,$(SUBDIRS))

TAR = cd ..; tar cvfz $(DIRNAME)-$@.tar.gz $(DIRNAME) -X xfiles

EXCLUDE_FROM_RELEASE =\
 */.* *.bak *.o *.out *.so *.a *.rej *.orig *~ \
 */CVS* \
 */autom4te.cache* \
 */templates.d* \
 */aclocal.m4 \
 */config.log \
 */config.status \
 $(DIRNAME)/config.mk \
 $(DIRNAME)/include/config.h \

deps: compile_idl_files cleandeps $(DEP_FILES)

cleandeps:
	rm -f $(DEP_FILES)

compile_idl_files:
	make -C idl compile_idl_files

$(DEP_FILES):
	echo "Remaking dependencies in $(@D)"
	cd $(@D) ; g++ $(CPPFLAGS) -MM *.cc \
	| sed 's%$(OMNIEVENTS_BASE)%..%' > $(@F)

src: deps
	make clean
	echo "$(EXCLUDE_FROM_RELEASE)" | tr ' ' '\n' > ../xfiles
	$(TAR)
	rm -f ../xfiles

# Builds a win32 binary release.
# Precondition: make must already have been run.

FindFiles = $(foreach ext, $(2), $(wildcard $(1)/*.$(ext)))

WIN32_TEXT_FILES = README LICENSE $(wildcard CHANGES*) examples/python/README
WIN32_FILES = \
 $(patsubst %,%.txt,$(WIN32_TEXT_FILES)) \
 omniEvents.exe \
 $(call FindFiles,examples,hh cc exe) \
 $(call FindFiles,examples/python,py) \
 $(call FindFiles,examples/python/*,py) \
 $(call FindFiles,idl,hh cc idl lib) \

win32: $(DIRNAME) $(patsubst %,$(DIRNAME)/%,$(WIN32_FILES))
	zip -r $(DIRNAME)-$@.zip $(DIRNAME)
	# rm -rf $(DIRNAME)

$(DIRNAME):
	install -d $(DIRNAME)

$(DIRNAME)/%.txt: %
	install -d `dirname $@`
	sed 's/$$/
/' $< > $@

$(DIRNAME)/omniEvents.exe: src/omniEvents.exe
	install -d `dirname $@`
	install $< $@

$(DIRNAME)/%: %
	install -d `dirname $@`
	install $< $@
