// -*- Mode: C++; -*-
//                            Package   : omniEvents
// EventChannelFactory_i.cc   Created   : 1/4/98
//                            Author    : Paul Nader (pwn)
//
//    Copyright (C) 1998 Paul Nader.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Description:
//      Implementation of the COSS Event Services Event Channel Factory
//	

#include "EventChannelFactory.h"

#include "Orb.h"
#include "EventChannel.h"
#include "omniEventsLog.h"

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

//------------------------------------------------------------------------
//           Event Channel Factory Interface Implementation
//------------------------------------------------------------------------
EventChannelFactory_i::EventChannelFactory_i(unsigned int port)
: Servant(Orb::inst()._omniINSPOA.in()),
  _port(port)
{
  activateObjectWithId("omniEvents");
}


EventChannelFactory_i::EventChannelFactory_i(
  unsigned int port,
  OepEcpsList& channels
)
: Servant(Orb::inst()._omniINSPOA.in()),
  _port(port)
{
  // Create event channels
  for(OepEcpsList::iterator i=channels.begin(); i!=channels.end(); i++)
  {
    EventChannel_i *channel =new EventChannel_i((*i)->getKey(),**i);
    channel->start();
    _eventChannels.insert(channel);
  }
  activateObjectWithId("omniEvents");
}


CORBA::Boolean
EventChannelFactory_i::supports(const CosLifeCycle::Key &k)
{
  if((k.length() == 1) &&
     (strcmp(k[0].id, "EventChannel") == 0) &&
     (strcmp(k[0].kind, "object interface") == 0))
    return 1;
  else
    return 0;
}


CORBA::Object_ptr
EventChannelFactory_i::create_object(
  const CosLifeCycle::Key &k,
  const CosLifeCycle::Criteria &criteria
)
{
  // Check the key
  if (! this->supports(k))
     throw CosLifeCycle::NoFactory(k);

  CORBA::String_var channelIdStr =newUniqueId();
  EventChannel_i* channel =new EventChannel_i(channelIdStr.in());

  // Process criteria
  for(CORBA::ULong i=0; i<criteria.length(); i++)
  {
    if(strcmp(criteria[i].name, "PullRetryPeriod") == 0)
    {
      CORBA::ULong pullRetryPeriod;
      if(! (criteria[i].value >>= pullRetryPeriod))
          throw CosLifeCycle::InvalidCriteria(criteria);
      if(pullRetryPeriod <= 0)
          throw CosLifeCycle::CannotMeetCriteria(criteria);
      channel->setPullRetryPeriod(pullRetryPeriod);
    }
    else if(strcmp(criteria[i].name, "MaxQueueLength") == 0)
    {
      CORBA::ULong maxQueueLength;
      if(! (criteria[i].value >>= maxQueueLength))
          throw CosLifeCycle::InvalidCriteria(criteria);
      if(maxQueueLength > 0)
          channel->setMaxQueueLength(maxQueueLength);
      else
          DB(1,"Ignoring CosLifeCycle criterion: MaxQueueLength=0");
    }
    else
        DB(1,"Ignoring unknown CosLifeCycle criterion: "<<criteria[i].name);
  }

  channel->start();
  _eventChannels.insert(channel);

  omniEventsLog::persist();
  return channel->_this();
}


void
EventChannelFactory_i::output(ostream &os)
{
  PortableServer::ObjectId_var oid=_poa->servant_to_id(this);
  CORBA::String_var oidStr =PortableServer::ObjectId_to_string(oid.in());

  os << "channelFactory\n{\n"
     << "\tPORT\t" << _port << "\n"
     << "\tKEY\t" << oidStr.in() << "\n";

  for(set<EventChannel_i*>::iterator i=_eventChannels.begin();
      i!=_eventChannels.end();
      ++i)
  {
     os << "\n";
     (*i)->output(os);
  }

  os << "\n}\n";
}


}; // end namespace OmniEvents
