// -*- Mode: C++; -*-
//                            Package   : omniEvents
// EventChannelFactory_i.h    Created   : 1/4/98
//                            Author    : Paul Nader (pwn)
//
//    Copyright (C) 1998 Paul Nader.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#ifndef OMNIEVENTS_EVENTCHANNELFACTORY_H
#define OMNIEVENTS_EVENTCHANNELFACTORY_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_IOSTREAM
#  include <iostream>
#else
#  include <iostream.h>
#endif

#include <set>

#include "EventChannelAdmin.hh"
#include "Servant.h"
#include "oep_types.h"

#ifdef HAVE_STD_STL
using namespace std;
#endif

namespace OmniEvents {

class EventChannel_i;

// Event Channel Factory

class EventChannelFactory_i :
  public virtual POA_EventChannelAdmin::EventChannelFactory,
  public Servant
{
public: // CORBA METHODS
  /** Returns true if the key passed has the following contents:
   *
   * - id   : "EventChannel"
   * - kind : "object interface"
   */
  CORBA::Boolean supports(const CosLifeCycle::Key& k);
  CORBA::Object_ptr create_object(
    const CosLifeCycle::Key &k,
    const CosLifeCycle::Criteria &the_criteria
  );

public:
  EventChannelFactory_i(unsigned int port);
  /** Builds an EventChannelFactory_i from the parsed logfile data. */
  EventChannelFactory_i(unsigned int port, OepEcpsList&);
  void output(ostream& os);

private:
  unsigned int         _port;
  set<EventChannel_i*> _eventChannels;
};

}; // namespace OmniEvents

#endif // OMNIEVENTS_EVENTCHANNELFACTORY_H
