//                            Package   : omniEvents
// Orb.cc                     Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Orb.h"

#ifdef HAVE_IOSTREAM
#  include <iostream>
#else
#  include <iostream.h>
#endif

#include <stdlib.h>

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

Orb Orb::_inst;

void Orb::init(int argc, char** argv)
{
  _orb=CORBA::ORB_init(argc,argv,"omniORB4");
  
  CORBA::Object_var obj;

  obj=_orb->resolve_initial_references("RootPOA");
  _RootPOA=PortableServer::POA::_narrow(obj);
  if(CORBA::is_nil(_RootPOA))
  {
    cerr<<"Failed to resolve initial reference: RootPOA."<<endl;
    exit(1);
  }

  obj=_orb->resolve_initial_references("omniINSPOA");
  _omniINSPOA=PortableServer::POA::_narrow(obj);
  if(CORBA::is_nil(_omniINSPOA))
  {
    cerr<<"Failed to resolve initial reference: omniINSPOA."<<endl;
    exit(1);
  }
  
  obj=_orb->resolve_initial_references("POACurrent");
  _POACurrent=PortableServer::Current::_narrow(obj);
  if(CORBA::is_nil(_POACurrent))
  {
    cerr<<"Failed to resolve initial reference: POACurrent."<<endl;
    exit(1);
  }
  
  obj=_orb->resolve_initial_references("NameService");
  _NameService=CosNaming::NamingContext::_narrow(obj);
  if(CORBA::is_nil(_NameService))
  {
    cerr<<"Failed to resolve initial reference: NameService."<<endl;
    exit(1);
  }
}


void Orb::run()
{
  while(true)
  {
    omni_thread::sleep(5);
    list<CORBA::Request_var>::iterator curr, next=_orphans.begin();
    while(next!=_orphans.end())
    {
      curr=next++;
      if((*curr)->poll_response())
      {
        CORBA::Environment_var env=(*curr)->env();
        if(!CORBA::is_nil(env) && env->exception())
        {
          CORBA::Exception* ex =env->exception(); // No need to free exception.
          DB(1,"Orphan call to "<<(*curr)->operation()<<"() got exception: "
              <<ex->_name()<<endl);
        }
        else
            DB(15,"Orphan call to "<<(*curr)->operation()<<"() returned.");
        _orphans.erase(curr);
      }
    }
  }
}


void Orb::orphanRequest(CORBA::Request* req)
{
  _orphans.push_back(req);
}


}; // end namespace OmniEvents
