//                            Package   : omniEvents
// Orb.h                      Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#ifndef OMNIEVENTS__ORB_H
#define OMNIEVENTS__ORB_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <list>
#include <omniORB4/CORBA.h>

#ifdef HAVE_STD_STL
using namespace std;
#endif

namespace OmniEvents {

/** Singleton class that owns the ORB and various initial references. */
class Orb
{
private:
  static Orb _inst;
  list<CORBA::Request_var> _orphans;

public:
  inline static Orb& inst()
  {
    return _inst;
  }

  CORBA::ORB_var               _orb;
  PortableServer::POA_var      _RootPOA;
  PortableServer::POA_var      _omniINSPOA;
  PortableServer::Current_ptr  _POACurrent;
  CosNaming::NamingContext_var _NameService;

  void init(int argc, char** argv);
  
  /** Parks the main thread, but also picks up (and ignores) responses from
   * orphan requests.
   */
  void run();
  
  /** Adopts the request and then stores it in _orphans.
   * run() later picks up and ignores the responses. This is a tidier
   * alternative to just calling req->send_oneway() - the ORB can get confused
   * by unexpected responses if we call twoway operations as oneway.
   */
  void orphanRequest(CORBA::Request* req);
};

/** Converts a string to a narrowed reference. */
template<class T>
typename T::_ptr_type string_to_(const char* oidStr)
{
  CORBA::Object_var obj =Orb::inst()._orb->string_to_object(oidStr);
  if(CORBA::is_nil(obj.in()))
    throw CORBA::BAD_PARAM();

  typename T::_var_type result =T::_narrow(obj);
  if(CORBA::is_nil(result.in()))
    throw CORBA::BAD_PARAM();

  return result._retn();
}

}; // end namespace OmniEvents

#endif // OMNIEVENTS__ORB_H
