//                            Package   : omniEvents
// ProxyPullSupplier.cc       Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "ProxyPullSupplier.h"
#include "Orb.h"
#include "omniEventsLog.h"

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

//
//  ProxyPullSupplierManager
//

PortableServer::Servant ProxyPullSupplierManager::incarnate(
  const PortableServer::ObjectId& oid,
  PortableServer::POA_ptr         poa
)
{
  // ?? Check that OID is valid here.
  ProxyPullSupplier_i* result =new ProxyPullSupplier_i(_managedPoa,_queue);
  _servants.insert(result);
  return result;
}

ProxyPullSupplierManager::ProxyPullSupplierManager(
  PortableServer::POA_ptr parentPoa,
  EventQueue&             q
)
: ProxyManager(parentPoa,"ProxyPullSupplier"),
  _queue(q)
{
  // pass
}

CosEventChannelAdmin::ProxyPullSupplier_ptr
ProxyPullSupplierManager::createObject()
{  
  return createNarrowedReference<CosEventChannelAdmin::ProxyPullSupplier>(
           _managedPoa,
           CosEventChannelAdmin::_tc_ProxyPullSupplier->id()
         );
}


//
//  ProxyPullSupplier_i
//

// CORBA interface methods

void ProxyPullSupplier_i::connect_pull_consumer(
  CosEventComm::PullConsumer_ptr pullConsumer
)
{
  try
  {
    if(CORBA::is_nil(pullConsumer))
        throw CORBA::BAD_PARAM();
    if(!CORBA::is_nil(_target) || !CORBA::is_nil(_req))
        throw CosEventChannelAdmin::AlreadyConnected();
    _target=CosEventComm::PullConsumer::_duplicate(pullConsumer);

    omniEventsLog::persist();
  }
  catch(...)
  {
    _target=CosEventComm::PullConsumer::_nil();
    deactivateObject();
    throw;
  }
}

void ProxyPullSupplier_i::disconnect_pull_supplier()
{
  DB(5,"ProxyPullSupplier_i::disconnect_pull_supplier()");
  deactivateObject();
  if(CORBA::is_nil(_target))
  {
    throw CORBA::BAD_INV_ORDER(
      omni::BAD_INV_ORDER_RequestUsedMoreThanOnce,
      CORBA::COMPLETED_YES
    );
  }
  else
  {
    CORBA::Request_var req=_target->_request("disconnect_pull_consumer");
    req->send_deferred();
    Orb::inst().orphanRequest(req._retn());
    _target=CosEventComm::PullConsumer::_nil();
  }
}

CORBA::Any* ProxyPullSupplier_i::pull()
{
  if(moreEvents())
      return new CORBA::Any(*nextEvent());
  else
      throw CORBA::TRANSIENT(
        omni::TRANSIENT_CallTimedout,
        CORBA::COMPLETED_NO
      );
}

CORBA::Any* ProxyPullSupplier_i::try_pull(CORBA::Boolean& has_event)
{
  if(moreEvents())
  {
    has_event=1;
    return new CORBA::Any(*nextEvent());
  }
  else
  {
    has_event=0;
    return new CORBA::Any();
  }
}

//

ProxyPullSupplier_i::ProxyPullSupplier_i(
  PortableServer::POA_ptr poa,
  EventQueue& q
)
: Proxy(poa),
  EventQueue::Reader(q),
  _target(CosEventComm::PullConsumer::_nil())
{
  // pass
}

void ProxyPullSupplier_i::reincarnate(const OEP_prxy& prxy)
{
  CosEventComm::PullConsumer_var pullConsumer =
    string_to_<CosEventComm::PullConsumer>(prxy.getIor());
  // Do not activate until we know that we have read a valid target.
  activateObjectWithId(prxy.getKey());
  connect_pull_consumer(pullConsumer.in());
}

void ProxyPullSupplier_i::output(ostream& os)
{
  Output out =basicOutput(os,"pxyPullSupplier",3,this,_target.in());
}


}; // end namespace OmniEvents
