//                            Package   : omniEvents
// ProxyPushSupplier.h        Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#ifndef OMNIEVENTS__PROXYPUSHSUPPLIER_H
#define OMNIEVENTS__PROXYPUSHSUPPLIER_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_IOSTREAM
#  include <iostream>
#else
#  include <iostream.h>
#endif

#include "ProxyManager.h"
#include "EventQueue.h"

#include "CosEventChannelAdmin.hh"

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

namespace OmniEvents {

class ProxyPushSupplierManager
: public ProxyManager,
  public omni_thread
{
public: // CORBA interface methods
  PortableServer::Servant incarnate(
    const PortableServer::ObjectId& oid,
    PortableServer::POA_ptr         poa
  );
  /** Pauses the thread, and then calls the parent's implementation. */
  void etherealize(
    const PortableServer::ObjectId& oid,
    PortableServer::POA_ptr         adapter,
    PortableServer::Servant         serv,
    CORBA::Boolean                  cleanup_in_progress,
    CORBA::Boolean                  remaining_activations
  );
public:
  ProxyPushSupplierManager(PortableServer::POA_ptr parentPoa,EventQueue& q);
  CosEventChannelAdmin::ProxyPushSupplier_ptr createObject();
  void* run_undetached(void*);

  omni_mutex     _lock;
  omni_condition _condition;
private:
  EventQueue&    _queue;
  omni_mutex     _conditionLock;
  bool           _shutdownRequested;
};


class ProxyPushSupplier_i
: public virtual POA_CosEventChannelAdmin::ProxyPushSupplier,
  public Proxy,
  public EventQueue::Reader
{
public: // CORBA interface methods
  void connect_push_consumer(CosEventComm::PushConsumer_ptr pushConsumer);
  void disconnect_push_supplier();
public:
  ProxyPushSupplier_i(PortableServer::POA_ptr poa, EventQueue& q);
  bool trigger();
  void reincarnate(const OEP_prxy& prxy);
  void output(ostream &os);
private:
  CosEventComm::PushConsumer::_var_type _target;
};


}; // end namespace OmniEvents

#endif // OMNIEVENTS__PROXYPUSHSUPPLIER_H
