//                            Package   : omniEvents
// SupplierAdmin.h            Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "SupplierAdmin.h"

#include "EventChannel.h"
#include "ProxyPushConsumer.h"
#include "ProxyPullConsumer.h"
#include "oep_types.h"

namespace OmniEvents {

CosEventChannelAdmin::ProxyPushConsumer_ptr
SupplierAdmin_i::obtain_push_consumer()
{
  if(!_pushConsumer)
      _pushConsumer=new ProxyPushConsumer_i(_poa,_queue);
  return _pushConsumer->createObject();
}


CosEventChannelAdmin::ProxyPullConsumer_ptr
SupplierAdmin_i::obtain_pull_consumer()
{
  if(!_pullConsumer)
      _pullConsumer=new ProxyPullConsumerManager(_poa,_queue);
  return _pullConsumer->createObject();
}


SupplierAdmin_i::SupplierAdmin_i(
  const EventChannel_i&   channel,
  PortableServer::POA_ptr poa
)
: Servant(poa),
  _channel(channel),
  _pushConsumer(NULL),
  _pullConsumer(NULL),
  _queue()
{
  activateObjectWithId("SupplierAdmin");
}


SupplierAdmin_i::SupplierAdmin_i(
  const EventChannel_i&   channel,
  PortableServer::POA_ptr poa,
  const OEP_saps&         saps
)
: Servant(poa),
  _channel(channel),
  _pushConsumer(NULL),
  _pullConsumer(NULL),
  _queue()
{
  OepPrxyList pdat;

  // Build Push Consumer proxies
  pdat = saps.getProxyPushConsumers();
  if(!pdat.empty())
  {
    _pushConsumer=new ProxyPushConsumer_i(_poa,_queue);
    for(OepPrxyList::iterator iter=pdat.begin(); iter!=pdat.end(); iter++)
    {
      _pushConsumer->reincarnate((*iter)->getKey(),(*iter)->getIor());
    }
  }

  // Build Pull Consumer proxies
  pdat = saps.getProxyPullConsumers();
  if(!saps.getProxyPullConsumers().empty())
  {
    _pullConsumer=new ProxyPullConsumerManager(_poa,_queue);
    _pullConsumer->reincarnate(saps.getProxyPullConsumers());
  }

  // Reactivate this servant.
  activateObjectWithId(saps.getKey());
}


SupplierAdmin_i::~SupplierAdmin_i()
{
  if(_pushConsumer)
  {
    delete _pushConsumer;
    _pushConsumer=NULL;
  }
  if(_pullConsumer)
  {
    delete _pullConsumer;
    _pullConsumer=NULL;
  }
  for(list<CORBA::Any*>::iterator i=_queue.begin(); i!=_queue.end(); ++i)
      delete *i;
}


void SupplierAdmin_i::collect(list<CORBA::Any*>& events)
{
  if(_pullConsumer)
     _pullConsumer->trigger();
  events=_queue;
  _queue.clear();
}


void SupplierAdmin_i::output(ostream& os)
{
  Output out =basicOutput(os,"supplierAdmin",2,this);
  if(_pushConsumer)
     _pushConsumer->output(os);
  if(_pullConsumer)
     _pullConsumer->output(os);
}


}; // end namespace OmniEvents
