// -*- Mode: C++; -*-
//                            Package   : omniEvents
// oep_types.cc               Created   : 1/10/99
//                            Author    : Paul Nader (pwn)
//
//    Copyright (C) 1998 Paul Nader.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Description:
//

/*
  $Log: oep_types.cc,v $
  Revision 1.2  2003/11/03 22:25:14  alextingle
  Updated License to GNU Lesser General Public v2.1

  Revision 1.1.1.1  2002/09/25 19:00:29  shamus13
  Import of OmniEvents source tree from release 2.1.1

  Revision 1.7  2000/10/03 08:38:21  naderp
  *** empty log message ***

  Revision 1.6  2000/09/26 08:47:15  naderp
  Configurable STL default parameters.

  Revision 1.5  2000/09/05 01:02:08  naderp
  Added MaxQueueLength QOS.

  Revision 1.4  2000/08/20 01:24:13  naderp
  R'ved incorrect cadm & sadm initialisation in OEP_ecps constructor.

  Revision 1.3  2000/03/06 13:14:34  naderp
  Removed name and added port.

  Revision 1.2  2000/03/02 01:28:01  naderp
  Moved proxy containment from event channel to admin objects.

  Revision 1.1  1999/11/02 08:00:57  naderp
  Using defaults.h

Revision 1.0  99/11/01  16:29:28  16:29:28  naderp (Paul Nader)
Initial revision

*/

#include "defaults.h"
#include "oep_types.h"

//------------------------------------------------------------------------
// Proxy Persistent Data Representation
//------------------------------------------------------------------------

OEP_prxy::OEP_prxy() :
  key(NULL),
  ior(NULL)
{
  // Empty
}

OEP_prxy::~OEP_prxy()
{
  if (key != NULL)
  {
    delete key;
  }

  if (ior != NULL)
  {
    delete ior;
  }
}

void
OEP_prxy::setKey(char *k)
{
  if (key != NULL)
  {
    delete key;
  }

  key = k;
}

const char *
OEP_prxy::getKey(void) const
{
  return key;
}

void
OEP_prxy::setIor(char *i)
{
  if (ior != NULL)
  {
    delete ior;
  }

  ior = i;
}

const char *
OEP_prxy::getIor(void) const
{
  return ior;
}

//------------------------------------------------------------------------
// Consumer Admin Persistent Data Representation
//------------------------------------------------------------------------

OEP_caps::OEP_caps() :
  key(NULL)
{
  // Empty
}

OEP_caps::~OEP_caps()
{
  if (key != NULL)
  {
    delete key;
  }

  OepPrxyList::iterator iter;

  for (iter=ppss.begin(); iter != ppss.end(); iter++)
  {
    if (*iter != NULL)
    {
      delete *iter;
    }
  }

  for (iter=ppls.begin(); iter != ppls.end(); iter++)
  {
    if (*iter != NULL)
    {
      delete *iter;
    }
  }

}

void
OEP_caps::setKey(char *k)
{
  if (key != NULL)
  {
    delete key;
  }

  key = k;
}

const char *
OEP_caps::getKey(void) const
{
  return key;
}

void
OEP_caps::addProxyPushSupplier(OEP_prxy *p)
{
  ppss.push_front(p);
}

const OepPrxyList &
OEP_caps::getProxyPushSuppliers(void) const
{
  return ppss;
}

void
OEP_caps::addProxyPullSupplier(OEP_prxy *p)
{
  ppls.push_front(p);
}

const OepPrxyList &
OEP_caps::getProxyPullSuppliers(void) const
{
  return ppls;
}

//------------------------------------------------------------------------
// Supplier Admin Persistent Data Representation
//------------------------------------------------------------------------

OEP_saps::OEP_saps() :
  key(NULL)
{
  // Empty
}

OEP_saps::~OEP_saps()
{
  if (key != NULL)
  {
    delete key;
  }

  OepPrxyList::iterator iter;

  for (iter=ppsc.begin(); iter != ppsc.end(); iter++)
  {
    if (*iter != NULL)
    {
      delete *iter;
    }
  }

  for (iter=pplc.begin(); iter != pplc.end(); iter++)
  {
    if (*iter != NULL)
    {
      delete *iter;
    }
  }
}

void
OEP_saps::setKey(char *k)
{
  if (key != NULL)
  {
    delete key;
  }

  key = k;
}

const char *
OEP_saps::getKey(void) const
{
  return key;
}

void
OEP_saps::addProxyPushConsumer(OEP_prxy *p)
{
  ppsc.push_front(p);
}

const OepPrxyList &
OEP_saps::getProxyPushConsumers(void) const
{
  return ppsc;
}

void
OEP_saps::addProxyPullConsumer(OEP_prxy *p)
{
  pplc.push_front(p);
}

const OepPrxyList &
OEP_saps::getProxyPullConsumers(void) const
{
  return pplc;
}

//------------------------------------------------------------------------
// Event Channel Persistent Data Representation
//------------------------------------------------------------------------


OEP_ecps::OEP_ecps() :
  key(NULL),
  pullRetryPeriod(PULL_RETRY_PERIOD),
  maxQueueLength(MAX_QUEUE_LENGTH),
  maxEventsPerConsumer(MAX_EVENTS_PER_CONSUMER)
{
  // Empty
}

OEP_ecps::~OEP_ecps()
{
  if (key != NULL)
  {
    delete key;
  }

  OepCapsList::iterator ci;

  for (ci=cadm.begin(); ci != cadm.end(); ci++)
  {
    if (*ci != NULL)
    {
      delete *ci;
    }
  }

  OepSapsList::iterator si;

  for (si=sadm.begin(); si != sadm.end(); si++)
  {
    if (*si != NULL)
    {
      delete *si;
    }
  }
}

void
OEP_ecps::setKey(char *k)
{
  if (key != NULL)
  {
    delete key;
  }

  key = k;
}

const char *
OEP_ecps::getKey(void) const
{
  return key;
}

void
OEP_ecps::addConsumerAdmin(OEP_caps *c)
{
  cadm.push_front(c);
}

const OepCapsList &
OEP_ecps::getConsumerAdmins(void) const
{
  return cadm;
}

void
OEP_ecps::addSupplierAdmin(OEP_saps *s)
{
  sadm.push_front(s);
}

const OepSapsList &
OEP_ecps::getSupplierAdmins(void) const
{
  return sadm;
}

void
OEP_ecps::setPullRetryPeriod(long p)
{
  pullRetryPeriod = p;
}

long
OEP_ecps::getPullRetryPeriod(void) const
{
  return pullRetryPeriod;
}

void
OEP_ecps::setMaxQueueLength(long m)
{
  maxQueueLength = m;
}

long
OEP_ecps::getMaxQueueLength(void) const
{
  return maxQueueLength;
}

void
OEP_ecps::setMaxEventsPerConsumer(long m)
{
  maxEventsPerConsumer = m;
}

long
OEP_ecps::getMaxEventsPerConsumer(void) const
{
  return maxEventsPerConsumer;
}



//------------------------------------------------------------------------
// Channel Factory Persistent Data Representation
//------------------------------------------------------------------------

OEP_cfps::OEP_cfps() :
  port(0),
  key(NULL)
{
  // Empty
}

OEP_cfps::~OEP_cfps()
{
  if (key != NULL)
  {
    delete key;
  }

  OepEcpsList::iterator iter;

  for (iter=channels.begin(); iter != channels.end(); iter++)
  {
    if (*iter != NULL)
    {
      delete *iter;
    }
  }
}

void
OEP_cfps::setPort(unsigned int p)
{
  port = p;
}

const unsigned int
OEP_cfps::getPort(void) const
{
  return port;
}

void
OEP_cfps::setKey(char *k)
{
  if (key != NULL)
  {
    delete key;
  }

  key = k;
}

const char *
OEP_cfps::getKey(void) const
{
  return key;
}

void
OEP_cfps::addChannel(OEP_ecps *channel)
{
  channels.push_front(channel);
}

OepEcpsList &
OEP_cfps::getChannels(void)
{
  return channels;
}
