// -*- Mode: C++; -*-
//                            Package   : omniEvents
// omniEventsLog.h            Created   : 1/10/99
//                            Author    : Paul Nader (pwn)
//
//    Copyright (C) 1998 Paul Nader.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Description:
//

/*
  $Log: omniEventsLog.h,v $
  Revision 1.4  2003/11/14 13:54:48  alextingle
  New output() members functions. Eliminates the need for friend ostream
  functions that are problematic on earlier versions of Microsoft
  VisualC++.

  Revision 1.3  2003/11/03 22:35:08  alextingle
  Removed all platform specific switches. Now uses autoconf, config.h.
  Added private helper functions initializeFileNames(), setFilename() &
  openOfstream() to simplify the implementation.
  Removed member `logdir', as it's only used during object construction.
  Renamed configuration macro LOGDIR_ENV_VAR to OMNIEVENTS_LOGDIR_ENV_VAR
  for consistency with other configuration macros.

  Revision 1.1.1.1.2.1  2002/09/28 22:20:51  shamus13
  Added ifdefs to enable omniEvents to compile
  with both omniORB3 and omniORB4. If __OMNIORB4__
  is defined during compilation, omniORB4 headers
  and command line option syntax is used, otherwise
  fall back to omniORB3 style.

  Revision 1.1.1.1  2002/09/25 19:00:32  shamus13
  Import of OmniEvents source tree from release 2.1.1

  Revision 1.3  2000/08/30 04:21:56  naderp
  Port to omniORB 3.0.1.

  Revision 1.2  2000/03/02 04:19:17  naderp
  Passing factory by reference to init() for initialisation.

  Revision 1.1  1999/11/02 13:40:56  naderp
  Rearranged data member definitions to avoid compiler warnings during
  initialisation.

  Revision 1.0  1999/11/01 16:48:21  naderp
  Initial revision

*/

#ifndef _OMNIEVENTSLOG_H_
#define _OMNIEVENTSLOG_H_

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_IOSTREAM
#  include <iostream>
#  include <fstream>
#else
#  include <iostream.h>
#  include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

#include <omniORB4/CORBA.h>

namespace OmniEvents {

class EventChannelFactory_i;

#ifndef OMNIEVENTS_LOGDIR_ENV_VAR
#  define OMNIEVENTS_LOGDIR_ENV_VAR "OMNIEVENTS_LOGDIR"
#endif

class omniEventsLog {

public:
   omniEventsLog(int port, const char* logdir=NULL);
   void init(EventChannelFactory_i *&);
   int getPort();
   void checkpoint(void);
   static void persist();
   void output(ostream& os);

public:
  class IOError {};

private:
  void initializeFileNames(const char* logdir);
  void setFilename(char*& filename,     const char* logdir,   const char* sep,
                   const char* logname, const char* hostname, const char* ext);
  void openOfstream(ofstream& s, const char* filename,
                    int modeflags=0, int* fd=NULL);

private:
  int port;
  ofstream logfile;
  int firstTime;
  char *active;				// Active file
  char *backup;				// Backup file
  char *checkpt;			// Checkpoint file
  omni_thread *recorder;                // In charge of checkpoints.
  EventChannelFactory_i *factory;
  int checkpointNeeded;
  static omniEventsLog *theLog;
  omni_mutex _lock;
};

class omniEventsLogWorker : public omni_thread {

public:
    typedef void (omniEventsLog::*Method)(void);
    omniEventsLogWorker(omniEventsLog *object,
                        Method method,
                        priority_t priority = PRIORITY_NORMAL);
    void* run_undetached(void *);
    ~omniEventsLogWorker();

private:
    omniEventsLog* _object;
    Method _method;
    omniEventsLogWorker();
};

}; // end namespace OmniEvents

#endif /* _OMNIEVENTSLOG_H_ */

