#                            Package   : omniEvents
# config.mk.in               Created   : 2003/10/31
#                            Author    : Alex Tingle
#
#    Copyright (C) 2003 Alex Tingle.
#
#    This file is part of the omniEvents application.
#
#    omniEvents is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    omniEvents is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

##
## MANUALLY SET THE BASE DIRECTORY FOR OMNIORB & OMNIEVENTS:
## 
OMNIORB_BASE    = L:\win32\src\omniORB-4.0.2
OMNIORB_LIBS    = omniORB402_rt.lib omniDynamic402_rt.lib omnithread30_rt.lib
OMNIEVENTS_BASE = L:\win32\omniEvents.testing_2_3_0
##

CORBA_ORB = omniORB4

# Punctuation
EMPTY :=
SPACE := $(EMPTY) $(EMPTY)

# Platform dependent file extensions.
OBJEXT          = obj
EXEEXT          = .exe
StaticLib       = $(1).lib

# Name of the application and libraries that we will build.
OMNIEVENTS      = omniEvents
CLIENT_LIB      = $(call StaticLib,$(OMNIEVENTS)Cl)

# for the link line...
ARCHIVES        = ../idl/$(CLIENT_LIB) \
                  ../util/$(call StaticLib,util) \
                  ../fe/$(call StaticLib,fe)

# Define installation directories.
prefix          = @prefix@
exec_prefix     = @exec_prefix@

INSTALL_BIN     = @bindir@
INSTALL_SBIN    = @sbindir@
INSTALL_LIB     = @libdir@
INSTALL_INCLUDE = @includedir@
INSTALL_IDL     = ${prefix}/share/idl

# Set up install & clean programs.
INSTALL = rem # No install on windows yet
RM      = del /F
RMDIR   = del /F

##
## IDL compiler.
##

IDL         = omniidl
IDL_COS_DIR = '$(OMNIORB_BASE)\idl'
IDLFLAGS    = -bcxx  # <--- backend selection should be omniidl's first option.
IDLFLAGS   += -Wba -Wbh='.hh' -Wbs='.cc' -Wbd='DynSK.cc' -WbBOA
IDLFLAGS   += -I$(IDL_COS_DIR)\COS

# The IDL compiler doesn't write code that uses config.h, so we have to compile
# the generated files with a couple of extra options:
EXTRA_IDL_CXXFLAGS = -D__WIN32__ -D__x86__=1 -D__OSVERSION__=5 \
                     -D_WIN32_WINNT=0x0400 -D__NT__ -D_COS_LIBRARY

##
## C++ compiler:  $(CXX) $(CPPFLAGS) $(CXXFLAGS) source
##

CXX       = cl -nologo
CXXFLAGS += -GX -MD -GR # CXXFLAGS

CPPFLAGS += -TP -I$(subst \,\\,$(OMNIORB_BASE))\\include # CPPFLAGS
CPPFLAGS += $(EXTRA_IDL_CXXFLAGS) -DHAVE_CONFIG_H # DEFS
CPPFLAGS += -I../idl -I../src
CPPFLAGS += -DENABLE_CLIENT_IR_SUPPORT=1 -DYY_NEVER_INTERACTIVE=1

##
## C++ Linker:  $(LD) objects $(LDFLAGS) $(LDOUT)target $(LDLIBS)
##

LD        = $(CXX)
LDFLAGS  += -GX -MD -GR # CXXFLAGS
LDFLAGS  += -link # LDFLAGS
LDLIBS   += /LIBPATH:$(subst \,\\,$(OMNIORB_BASE))\\lib\\x86_win32 \
  ws2_32.lib mswsock.lib advapi32.lib $(OMNIORB_LIBS) # LIBS

##
## Command to build C++ static archive (library)
## $(1) - archive/library (output) filename
## $(2) - objects and static libraries to include
##

CxxBuildStatic = link /lib /nologo /OUT:$(1) $(2)

##
## Command to build C++ executable.
## $(1) - executable (output) filename
## $(2) - objects and static libraries to include
##

CxxBuildExecutable = $(LD) $(2) $(LDFLAGS) /OUT:$(1) $(LDLIBS)

##
## Some compilers & linkers need to be told where to store
## template instantiations.
##

CXXFLAGS_PTR =
CXX_REPOSITORY = $(OMNIEVENTS_BASE)/templates.d
ifneq ("$(CXXFLAGS_PTR)","")
  CXXFLAGS += $(CXX_REPOSITORY)
  LDFLAGS  += $(CXX_REPOSITORY)
endif

##
## Suffix rules for compiling .idl & .cc files.
##

.SUFFIXES: .idl .cc .$(OBJEXT)

.idl.cc:
	$(IDL) $(IDLFLAGS) $<

.cc.$(OBJEXT):
	$(CXX) -c $(CPPFLAGS) $(CXXFLAGS) $<

# (end)
